
;*** LCD-DISPLAY Routinen ********************************************

;*********************************************************************
;  LCD-Init
;
;  Scratch-Reg: TEMP_A, ZL, ZH
;*********************************************************************

lcd_init:
        rcall   lcd_reset

        ldi     TEMP_A,$28                  ; 4Bit - Mode, 2 Zeilen
        rcall   lcd_control

        ldi     TEMP_A,$80                  ; DDRAM auf Anfang
        rcall   lcd_control

        ldi     TEMP_A,$01                  ; Display lschen
        rcall   lcd_control

		rcall	wait_5ms

        ldi     TEMP_A,$06                  ; Increment, Not Shiftet
        rcall   lcd_control

        ldi     TEMP_A,$10                  ; Cursor Move, Shift off
        rcall   lcd_control

        ldi     TEMP_A,$0C                  ; Display an, Cursor aus
        rcall   lcd_control

        ret

;*********************************************************************
;  Write LCD-Reset...  keine Daten, kein Busy
;
;  Scratch-Reg: TEMP_A
;*********************************************************************

lcd_reset:
		rcall	wait_5ms
		rcall	wait_5ms
		rcall	wait_5ms

		cbi     LCD_CTRL,LCD_RS			; Register auf Control

		ldi		TEMP_A,$03				; D4...7 an Bit 0...3
		out		LCD_OUT,TEMP_A

		rcall	lcd_clock

		rcall	wait_5ms

		rcall	lcd_clock

		rcall	wait_5ms

		rcall	lcd_clock

		rcall	wait_5ms

        ldi     TEMP_A,$02                  ; 4Bit - Mode
        rcall   lcd_clock

		rcall	wait_5ms

        ret

;*********************************************************************
;  Write LCD-Command...  Daten in TEMP_A, T-Flag=0 Control, T-Flag=1 Daten
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_control:
		cbi     LCD_CTRL,LCD_RS			; Register auf Control
		rjmp	lcd_output

;*********************************************************************
;  Write LCD-Command...  Daten in TEMP_A, T-Flag=0 Control, T-Flag=1 Daten
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_daten:
		sbi     LCD_CTRL,LCD_RS			; Register auf Control

lcd_output:
		push    TEMP_B
		clr		TEMP_B					; warten auf Display rund 50s bei 16MHz

lcd_daten_loop_1:
		dec		TEMP_B
		brne	lcd_daten_loop_1

lcd_daten_loop_2:
		dec		TEMP_B
		brne	lcd_daten_loop_2

        pop     TEMP_B

		push	TEMP_A					; merken
		swap	TEMP_A
		andi	TEMP_A,0b00001111		; oberes Nibble
		out		LCD_OUT,TEMP_A

		rcall	lcd_clock

		pop		TEMP_A					; Wert zurckholen
		andi	TEMP_A,0b00001111		; nur High-Nibble
		out		LCD_OUT,TEMP_A

		rcall	lcd_clock

        ret

;*********************************************************************
;  Clock LCD E-Impuls
;
;  Scratch-Reg: ---
;*********************************************************************

lcd_clock:

        sbi     LCD_CTRL,LCD_E          ; E auf 1		; 750nS
        nop
		nop
        nop
		nop
        nop
		nop
		cbi     LCD_CTRL,LCD_E          ; E auf 0

		ret

;*********************************************************************
;  Anzeigen...   immer beide Zeilen des Displays schreiben
;
;  Scratch-Reg: TEMP_A, TEMP_B, ZL, ZH
;*********************************************************************

lcd_write:
		load_p	Z,LCD_LINE_1

		ldi		TEMP_A,ZEILE_1
		rcall	lcd_control

        set                             ; setzt T fr Daten in lcd_control
        ldi     TEMP_B,LCD_ZEILE        ; Anzahl Zeichen
lcd_write_zeile_1_loop:
        ld      TEMP_A,Z+               ; erstes Byte
        rcall   lcd_daten
        dec     TEMP_B
        brne    lcd_write_zeile_1_loop

 		ldi		TEMP_A,ZEILE_2
		rcall	lcd_control

        set                             ; setzt T fr Daten in lcd_control
        ldi     TEMP_B,LCD_ZEILE        ; Anzahl Zeichen

lcd_write_zeile_2_loop:
        ld      TEMP_A,Z+               ; erstes Byte
        rcall   lcd_daten
        dec     TEMP_B
        brne    lcd_write_zeile_2_loop

        clt                             ; lscht T fr CTRL

        ret


